#!/bin/env dls-python

import read_config as cfg
import importlib
import socket
import time
import os
import logging

class RunTests(object):
    def __init__(self, configpath, hostname, cmdport):
        self.file_logger = logging.getLogger('filelog')
        self.console_logger = logging.getLogger('consolelog')
        self.hostname = hostname
        self.config = self.get_config(configpath)
        self.test_modules = self.import_modules()
        try:
            self.cmdsock = self.setup_connection(hostname, cmdport)
        except socket.error as e:
            self.file_logger.error(e)

    def setup_connection(self, hostname, cmdport):
        try:
            cmdsock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            cmdsock.connect((hostname, cmdport))
            cmdsock.setsockopt(socket.SOL_TCP, socket.TCP_NODELAY, 1)
            cmdsock.settimeout(1)
        except socket.error:
            self.file_logger.error("Problem with connection to {}".format(self.hostname))
            raise
        return cmdsock

    def get_config(self, path):
        cfgread = cfg.ConfigReader()
        return cfgread.parse_config_file(path)

    def import_modules(self):
        modules = {}
        for key, name in self.config.iteritems():
            try:
                modules[name[0][0]] = importlib.import_module('tests.' + name[0][0])
                # modules.append(importlib.import_module('tests.' + name[0][0]))
            except ImportError as e:
                self.file_logger.error("MODULE DOES NOT EXIST, SKIPPING:  {}".format(name))
                del self.config[name[0][0]]
        return modules

    def run_tests(self):
        timestr = time.strftime("%Y%m%d-%H%M%S")
        cls_list = []

        for name, config_list in self.config.items():
            try:
                test_cls = getattr(self.test_modules[name], name)
                for config in config_list:
                    self.file_logger.info('-- {:<7} {}'.format(
                        name.upper(), ''.join(['-']*24)
                    ))
                    tst = test_cls(self.cmdsock, timestr, config[1])
                    cls_list.append(tst)
                    #restore the panda state to make sure we have a known start point
                    os.system(
                        "./load-state {} base.state".format(self.hostname))
                    cls_list[-1].run()
                    cls_list[-1].get_summary()
            except AttributeError:
                self.file_logger.error("MODULE DOES NOT EXIST, SKIPPING:  {}".format(name))

        self.console_logger.info('-- SUMMARY {}'.format(''.join(['-'] * 24)))
        self.console_logger.info('{:<10}{:<11}{:<10}'.format('TEST', 'NUMBER', 'ERRORS'))
        self.console_logger.info('{}'.format(''.join(['-'] * 35)))
        for cls in cls_list:
            summary = cls.get_summary()
            self.console_logger.info(summary)

def setup_logging(logPath):
    file_format_string = "%(asctime)s [%(levelname)-5s] [%(name)-10s]  %(message)s"
    console_format_string = "%(asctime)s [%(levelname)-5s] %(message)s"
    logging.basicConfig(level=logging.DEBUG,
                        format=file_format_string,
                        filename=logPath)

    logFormatter = logging.Formatter(console_format_string)
    #set to log to console
    console_handler = logging.StreamHandler()
    console_handler.setFormatter(logFormatter)

    console_logger = logging.getLogger('consolelog').addHandler(console_handler)
    file_logger = logging.getLogger('filelog')


def view_log(log_file_path):
    inputy = '.'
    while inputy.strip('\n') not in ['y', 'n']:
        print "\nView logfile: {} [y/n]".format(log_file_path)
        inputy = raw_input()
    if inputy.strip('\n') in ['y']:
        f = open(log_file_path, 'r')
        file_contents = f.read()
        print (file_contents)
        f.close()


def main():
    timestr = time.strftime("%Y%m%d-%H%M%S")
    log_file_name = os.path.join('log', "{}.log".format(timestr))
    setup_logging(log_file_name)
    rt = RunTests('config', '172.23.252.202', 8888) #pass in as parameter
    rt.run_tests()
    view_log(log_file_name)

if __name__ == '__main__':
    main()